@@ -20,10 +20,14 @@ from .qestatistics import g_stat
 from .qebacktestmul import runBacktest, historyDataBackTest, dynamicBacktest
 from .qeredisdb import savePidToDB,saveStrategyfreqToDB,initMyredis
 #from .qecsvorder import qeCsvOrders
-from .qeglobal import  getAccidTraderQueue, setClassAccID, getExemode
+from .qeglobal import  getAccidTraderQueue, setClassAccID
 from .qesysconf import read_sysconfig
 #from .qecsvorder import g_csvorders
-
+try:
+    from .qestockmarket import runStockMarketProcess, checkStockTime
+except:
+    checkStockTime =lambda x: False
+    runStockMarketProcess = lambda x1,x2,x3,x4: None
 
 try:
     from codeconfig import getSoptFuncs
@@ -115,7 +119,9 @@ def processFunc(processInfo):
             elif processInfo['name'].find('sopt') >= 0 and soptmd:
                 soptmd(curuser,'888888',processInfo['strats'],processInfo['runmode'], None)
                 logger.info("add sopt Market")
-                
+            elif processInfo['name'].find('stock') >= 0:
+                runStockMarketProcess(curuser,'888888',processInfo['strats'],processInfo['runmode'])
+                logger.info("add stock Market") 
             
         elif processInfo['type'] == 'trader':
             logger.info("add simulation Trader")
@@ -160,7 +166,7 @@ def getAddress(user, mode, account, rfrate=0.02):
     # webaddress = socket.gethostbyname(socket.gethostname())
     ip = '127.0.0.1:5000'
     try:
-        from .qedockerconfig import log_path
+        #from .qedockerconfig import log_path
         with open('/srv/jupyterhub/webhost', 'r') as f:
             ip = f.read()
             if ip[-1] == '\n' or ip[-1] == '\t' or ip[-1] == '\r':
@@ -273,6 +279,8 @@ def startSimuProcess(user, token, strats, feesmult=1.0, ignorepass=True, mode_72
                 markettime = True
             if ('sopt' in md or 'sopttest' in md) and sopttime and sopttime(now):
                 markettime = True
+            if 'stock' in md and checkStockTime(now):
+                markettime = True
             if not markettime:    
                 print(u"市场此时没有开盘.")
                 return
@@ -460,6 +468,7 @@ def startRealProcess(user, strats, user_setting, feesmult=1.0, ignorepass=True,
     evalmode = False
     for j in range(len(user_setting)):
         sett = user_setting[j]
+        
         brokerinfo = get_broker_info(sett['broker'])
         if not 'broker' in sett or brokerinfo is None:
             print("检查user_setting的'broker'不合法")
@@ -493,7 +502,7 @@ def startRealProcess(user, strats, user_setting, feesmult=1.0, ignorepass=True,
         #    return
         else:
             classset.add(sett['class'])
-            accounts.append(realAccountInfo(accid=j))
+            accounts.append(realAccountInfo(user=user, token=user_setting[j]['investorid'],accid=j))
             setClassAccID(sett['class'], j)
         apiset.add(sett['api'])
         if sett['api'].find('test') > 0:
@@ -538,6 +547,8 @@ def startRealProcess(user, strats, user_setting, feesmult=1.0, ignorepass=True,
                 markettime = True
             if ('sopt' in apiset or 'sopttest' in apiset) and sopttime and sopttime(now):
                 markettime = True
+            if 'stock' in apiset and checkStockTime(now):
+                markettime = True
             if not markettime:    
                 print(u"市场此时没有开盘.")
                 return
