@@ -12,6 +12,7 @@ from qesdk import get_package_address
 import requests
 import shutil
 import zipfile
+import glob
 
 def get_mac_address():
     import uuid
@@ -72,8 +73,13 @@ def installPlugin(plugin_name, version='latest'):
             zip_ref.extractall(global_pluginpath)
 
         print("解压缩完成！")
-        
-        
+        suffix = '*.pyd' if global_platform== 'windows' else '*.so'
+        old_files = glob.glob(os.path.join(global_pluginpath, suffix))
+        if old_files:
+            print('删除旧版文件...')
+        for old_file in old_files:
+            os.chmod(old_file, 0o777)
+            os.remove(old_file)
         print(f'插件{plugin_name}下载成功')
         print('在策略文件中按如下格式import该插件:')
         print(f'from qetrader.plugins.qe{plugin_name} import plugin_{plugin_name}')
