@@ -24,6 +24,7 @@ class qeEnumTypes():
     KEY_TIMER = 15
     KEY_TIMER_PROCESS = 16
     KEY_TIMER_SIMU = 17
+    KEY_MARKET_MULTIDATA = 18
     KEY_SEND_ORDER = 101
     KEY_CANCEL_ORDER = 102
     KEY_STATUS_ALL_TRADED = "alltraded"
@@ -33,6 +34,7 @@ class qeEnumTypes():
     KEY_STATUS_UNKNOWN = 'unknown'
     KEY_STATUS_REJECT = 'failed'
     KEY_STATUS_PTPC = 'ptpc'
+    KEY_STATUS_CANCEL_FAILED = 'cacelfailed'
     KEY_REQ_ACCOUNT = 110
     KEY_REQ_POSITION = 111
     KEY_REQ_POSITION_DETAIL = 112
@@ -40,6 +42,7 @@ class qeEnumTypes():
     KEY_QRY_ORDER = 114
     KEY_QRY_TRADE = 115
     KEY_USER_LOGOUT = 116
+    KEY_ON_CANCEL_CONFIRM = 117
 
 qetype = qeEnumTypes()
 
